@@ -8,6 +8,8 @@ from vivarium.core.process import (
 from vivarium.core.composer import Composer, MetaComposer, Composite
 from vivarium.core.types import State, Schema, Update, Topology
 from vivarium.processes.division import get_divide_update
+from vivarium.core.engine import Engine
+
 
 quark_colors = ['green', 'red', 'blue']
 quark_spins = ['up', 'down']
@@ -632,6 +634,25 @@ def test_override() -> None:
     assert default_state == expected_default_state
 
 
+def test_composer() -> Dict:
+    toy_compartment = ToyCompartment({})
+    total_time = 10
+    initial_state = {
+        'periplasm': {
+            'GLC': 20,
+            'MASS': 100,
+            'DENSITY': 10},
+        'cytoplasm': {
+            'GLC': 0,
+            'MASS': 3,
+            'DENSITY': 10}}
+
+    composite = toy_compartment.generate()
+    sim = Engine(composite=composite, initial_state=initial_state)
+    sim.update(total_time)
+    return sim.emitter.get_timeseries()
+
+
 def test_composite_initial_state() -> None:
     """
     test that initial state in composite merges individual processes'
